'use strict';

const signedFetch = require('signed-aws-es-fetch');
const logger = require('@financial-times/n-logger').default;

function compat(tag) {
	return {
		term: {
			name: tag.prefLabel,
			attributes: tag.attributes,
			id: tag.idV1,
			taxonomy: tag.taxonomy
		}
	};
}

function getTag(id, type) {
	const data = {
		_source:[ "metadata"],
		query: {
			term: {}
		},
		sort: {
			publishedDate: {
				order: 'desc'
			}
		},
		size: 1
	};
	data.query.term[`metadata.${type}`] = id;

	return signedFetch('https://next-elastic.ft.com/v3_api_v2/item/_search', {
		method: 'POST',
		timeout: 3000,
		headers: {
			'Content-Type': 'application/json'
		},
		body: JSON.stringify(data)
	})
	.then(res => {
		if (res.ok) {
			return res.json();
		}
		throw new Error("Can't find tag");
	})
	.then(data => {
		if (data.hits.hits[0]) {
			return {
				status: 200,
				body: compat(data.hits.hits[0]._source.metadata.find(tag => tag[type] === id))
			};
		}
		return {
			status: 404,
			body: { error: 'Not found, could be deleted or might never had existed' }
		};
	})
	.catch(() => {
		return {
			status: 500,
			body: { error: 'Server error' }
		};
	});
}

module.exports = (opts) => {
	const identifierValues = opts.identifierValues;
	const identifierType = opts.identifierType || 'idV1';

	if(identifierValues.length === 0 || !Array.isArray(identifierValues)) {
		return Promise.resolve([]);
	}

	return Promise.all(
		identifierValues.map(id => getTag(id, identifierType))
	).then(results => {
		const items = results
					.filter(r => r.status === 200)
					.map(r => r.body.term);
		return {
			total: items.length,
			items
		};
	})
	.catch((err) => {
		logger.warn('Failed getting ElasticSearch things', {
			identifierValues,
			error: err
		});
	});
};
